/*******************************************************************************




     Alarme Residencial Temporizado

     C/ PIC 12F675

     Sensores com Reed Switch e im

     Autor: Cludio Lrios

     Data: 28/02/2017

     Objetivo: Alarme residencial usando sensores reed switch instalados nas
     portas e janelas.
     Uso: Feche todas as janelas e portas com sensores. Ligue o interruptor de
     liga/desliga.
     Caso ao ligar o alarme comear a tocar a sirene, ento algum sensor est
     acionado (alguma porta ou janela aberta). 
     Aps ser ligado o alarme, passar a contar um tempo para que o usurio saia
     do recinto e  feche a porta de sada.
     As portas e janelas ligadas a entrada sem temporizao (ST) disparam
     imediatamente ao serem abertas, aps o tempo inicial para sair do recinto.
     A porta usada para entrar e sair dever ser ligada na entrada temporizada
     (CT).
     Isto permite que ao entrar na casa, o usurio  desligue o alarme dentro de
     um tempo programado, sem que ocorra o disparo.
     Se a funo AVISO_P_DESLIGAR estiver definida, a sirene dar um breve 
     toque ao abrir a porta, avisando para desligar o alarme antes que dispare.


     Uso didtico

     Este arquivo  parte integrante do blog Lrios.Tecnologia.WS

 *****************************************************************************/

#include <12F675.h>
#device adc=8
#use delay(clock=4000000)
#fuses NOWDT,INTRC_IO, NOCPD, NOPROTECT, NOMCLR, NOPUT, NOBROWNOUT//configurao

//bits
#bit GPIF          = 0x0b.0  //flag da int. mudana
#bit in_ct         = 0x05.4  //entrada temporizada pino 3
#bit in_st         = 0x05.5  //entrada sem temporizao pino 2
#bit sirene        = 0x05.2  //sada para ligar sirene
#bit tmr0if        = 0x0b.2  //flag de overflow do timer 0

//bytes
#byte OPTION_REG = 0x81
#byte TRISIO = 0x85
#byte GPIO   = 0x05
#byte INTCON  = 0x0B
#byte TMR0    = 0x01

//******************************************************************************
//definies do usurio
//******************************************************************************
//defina o tempo para a pessoa sar do recinto aps ligar o alarme
#define  TEMPO_P_SAIR  45  //tempo (segundos) para sair do recinto
//******************************************************************************
//defina o tempo para pessoa desligar o alarme aps entrar na casa
#define  TEMPO_AO_ENTRAR 30 //tempo (segundos) ao entrar (entrada temporizada)
//******************************************************************************
//opo para disparo direto ou por um tempo
#define  DISPARA_N_MINUTOS  //se descomentado, tocar por 15 minutos e parar
//******************************************************************************
//defina abaixo o tempo que deseja que a sirene fique disparada
#define  TEMPO_DISPARADO 15   //tempo (minutos) em que a sirene fica ligada
//******************************************************************************
//opo de aviso para desligamento do alarme ao entrar -  1 toque
#define  AVISO_P_DESLIGAR //descomente para habilitar e comente para inibir
//******************************************************************************
//opo de aviso quando acaba o tempo inicial - 2 toques
#define AVISO_ALARME_ARMADO //descomente para habilitar e comente para inibir
//opo de aviso que o alarme est operando ao ligar interruptor - 1 toque
#define AVISO_ALARME_LIGOU

/*******************************************************************************
               SUBROTINAS
*******************************************************************************/



/******************************************************************************
               ROTINA PRINCIPAL (MAIN)
*******************************************************************************/

void main() {
   unsigned int8 a,b;
   setup_adc_ports(NO_ANALOGS);  //seleciona AN2 e AN3 com 5v como referncia
   setup_counters(RTCC_INTERNAL,RTCC_DIV_1);
   setup_timer_1(T1_DISABLED);
   setup_comparator(NC_NC_NC_NC);
   setup_vref(FALSE);
   TRISIO= 0b111011;//somente pinos 5 como sada
   gpio=0;
   delay_ms(500);//delay 
    
   #ifdef AVISO_ALARME_LIGOU
   sirene=1;//liga sirene por um curto periodo para lembrar de desligar
   delay_ms(300);//delay curto
   sirene=0;//desliga
   #endif
   if(in_st)sirene=1;//se ao ligar existir uma porta/janela sem fechar, ento dispara
   
   for(a=0;a<TEMPO_P_SAIR;a++) delay_ms(1000);
    #ifdef AVISO_ALARME_ARMADO
         sirene=1;//liga sirene por um curto periodo para lembrar de desligar
         delay_ms(500);//delay curto
         sirene=0;//desliga
         delay_ms(500);//delay curto
         sirene=1;//liga sirene por um curto periodo para lembrar de desligar
         delay_ms(500);//delay curto
         sirene=0;//desliga
    #endif
/******************************************************************************
              LAO DE REPETIO
*******************************************************************************/

   for(;;){

     enable_interrupts(int_ra5|int_ra4);//liga interrupo por mudana nos pinos 2 e 3
     GPIO=0;//desliga sada
     GPIF=0;//reseta flag da mudana de estado
     sleep();//entra em modo de baixo consumo
     delay_us(1);//delay ps sleep
     delay_ms(50);//debouncing dos interruptores (reed switch)
     if(in_ct){
         #ifdef AVISO_P_DESLIGAR
         sirene=1;//liga sirene por um curto periodo para lembrar de desligar
         delay_ms(500);//delay curto
         sirene=0;//desliga
         #endif
        for(a=0;a<TEMPO_AO_ENTRAR;a++) delay_ms(1000);//conta tempo para usurio entrar
        sirene=1;//liga sirene aps este tempo
     }
     if(in_st)sirene=1;//liga sirene de imediato (entrada sem temporizao)

     if(sirene){
     #ifdef DISPARA_N_MINUTOS
     for(b=0;b<TEMPO_DISPARADO;b++){  //desliga sirene disparada aps um tempo
       for(a=0;a<60;a++) delay_ms(1000);//gera ~ 1 minuto 
     }
     sirene=0;//desliga sirene aps o tempo acima 
     #else
     for(;;); //lao eterno, mantendo sirene ligada at se desligar a alimentao
     #endif
     }

   }//for(;;);
}//main()
//-----------------------------------------fim----------------------------------
